/**********************************************************************
*   Copyright: (C)2021 LingYun IoT System Studio <www.weike-iot.com>
*      Author: GuoWenxue<guowenxue@gmail.com> QQ: 281143292
* Description: Common AT command source code
*
*
*   ChangeLog:
*        Version    Date       Author            Description
*        V1.0.0  2022.05.1  GuoWenxue      Release initial version
***********************************************************************/

#ifndef INC_ATCMD_H_
#define INC_ATCMD_H_


#include "usart.h"

#define EXPECT_OK    "OK\r\n"

/* 函数说明：给模块发送一个AT命令，并在超时时间内判断是否收到所期待的字符串。如果传入了一个buf，则将收到的AT命令回复数据通过 obuf 返回出去。
 * 输入参数：        $atcmd: 要发送的 AT 命令
 *        $expect_reply: 发送AT命令期待的回复，如果不指定则默认等待 "OK\r\n";
 *             $timeout: 等待接收AT命令回复数据的超时时间
 *
 * 输出参数：
 *                $obuf: 接收AT命令回复数据的buf，如果传 NULL 则不会返回给调用函数；
 *                $size: 接收buf的大小
 *
 *   返回值：    -1: 输入参数出错；   0: AT命令收到期待的字符串   -2: 接收AT命令回复超时    -3: 收到 ERROR 等字符串
 *
 */
extern int send_atcmd(char *atcmd, char *expect_reply, unsigned int timeout, char *obuf, int size);


/*
 *  Description: Send AT command which will only reply by "OK" or "ERROR", such as AT:
 *                 Reply:   \r\nOK\r\n
 * Return Value: 0: OK     -X: Failure
 */
extern int send_atcmd_check_ok(char *atcmd, unsigned int timeout);


/*
 *  Description: Send AT command which will reply by a value directly in a single line, such as AT+CGMM:
 *                  Reply:   \r\nEC20F\r\nOK\r\n
 *
 * Return Value: 0: OK     -X: Failure
 */
extern int send_atcmd_check_value(char *atcmd, unsigned int timeout, char *reply, int size);


/*
 *  Description: Send AT command which will reply by the value  with a prefix "+CMD: " line, such as AT+CSQ:
 *                  Reply:   \r\n+CSQ: 26,99\r\nOK\r\n
 *
 * Return Value: 0: OK     -X: Failure
 */
extern int send_atcmd_check_request(char *atcmd, unsigned int timeout, char *reply, int size);

/* 函数说明： 给模块发送"AT"命令，确认模块是否正常回复。
 * 输入参数： $times: 尝试发送 AT 命令测试的次数
 *  返回值： 0: 发送AT命令收到OK回复     <0: 发送AT命令等待回复失败。
 */
extern int send_atcmd_check(int times);


/* 函数说明： 给模块发送"ATE0"关闭回显，或 "ATE1" 打开回显。
 * 输入参数： $on: 打开还是关闭回显,ECHO_OFF 或  ECHO_ON
 *  返回值： 0: 发送成功     <0: 发送失败。
 */
enum
{
	ECHO_OFF,
	ECHO_ON,
};
extern int send_atcmd_echo(int status);

#endif /* INC_ATCMD_H_ */
