/**********************************************************************
*   Copyright: (C)2021 LingYun IoT System Studio <www.weike-iot.com>
*      Author: GuoWenxue<guowenxue@gmail.com> QQ: 281143292
* Description: Common AT command source code
*
*
*   ChangeLog:
*        Version    Date       Author            Description
*        V1.0.0  2022.05.1  GuoWenxue      Release initial version
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "usart.h"
#include "atcmd.h"

#define ATCMD_REPLY_LEN          256


//#define CONFIG_ATCMD_DEBUG  /* AT命令相关函数调试开关 */

#ifdef CONFIG_ATCMD_DEBUG
#define atcmd_dbg(format,args...) printf(format, ##args)
#else
#define atcmd_dbg(format,args...) do{} while(0)
#endif


/* AT command */

int send_atcmd(char *atcmd, char *expect_reply, unsigned int timeout, char *obuf, int size)
{
	int                       rv = -3;
	unsigned int              i;
	char                     *expect;

	/* check function input arguments validation */
	if(  !atcmd || strlen(atcmd)<=0 )
	{
		atcmd_dbg("ERROR: Invalid input arguments\r\n");
		return -1;
	}

	atcmd_dbg("\r\nStart send AT command: %s", atcmd);
	clear_atcmd_buf();
	HAL_UART_Transmit(atcmd_huart, (uint8_t *)atcmd, strlen(atcmd), 1000);

	expect = expect_reply ? expect_reply : "OK\r\n";

	/* Receive AT reply string by UART interrupt handler, stop by "OK/ERROR" or timeout */
	for(i=0; i<timeout; i++)
	{
		if( strstr(g_atcmd_rxbuf, expect) )
		{
			atcmd_dbg("AT command Got expect reply '%s'\r\n", expect);
			rv = 0;
			goto CleanUp;
		}

		if( strstr(g_atcmd_rxbuf, "ERROR\r\n") || strstr(g_atcmd_rxbuf, "FAIL\r\n"))
		{
			rv = -2;
			goto CleanUp;
		}

		HAL_Delay(1);
	}


CleanUp:
	if( obuf && size>0 )
	{
		strncpy(obuf, g_atcmd_rxbuf, size);
	}
	atcmd_dbg("<<<< AT command reply:\r\n%s", g_atcmd_rxbuf);
	return rv;
}


/*
 *  Description: Send AT command which will only reply by "OK" or "ERROR", such as AT:
 *                 Reply:   \r\nOK\r\n
 * Return Value: 0: OK     -X: Failure
 */
int send_atcmd_check_ok(char *atcmd, unsigned int timeout)
{
    if( !atcmd )
    {
    	atcmd_dbg("Invalid input arguments\r\n");
        return -1;
    }

    return send_atcmd(atcmd, EXPECT_OK, timeout, NULL, 0);
}


/*
 *  Description: Send AT command which will reply by a value directly in a single line, such as AT+CGMM:
 *                  Reply:   \r\nEC20F\r\nOK\r\n
 *
 * Return Value: 0: OK     -X: Failure
 */
int send_atcmd_check_value(char *atcmd, unsigned int timeout, char *reply, int size)
{
    int                     i = 0;
    int                     rv;
    char                    buf[ATCMD_REPLY_LEN];
    char                   *ptr_start, *ptr_end;

    if( !atcmd || !reply || size<=0 )
    {
    	atcmd_dbg("Invalid input arguments\r\n");
        return -1;
    }

    /*+-----------------------+
     *|    send AT command    |
     *+-----------------------+
     */
    rv = send_atcmd(atcmd, EXPECT_OK, timeout, buf, sizeof(buf));
    if( rv < 0 )
    {
        return -2;
    }

    /*+-----------------------+
     *|  Find start poistion  |
     *+-----------------------+
     */
    ptr_start=strchr(buf, '\n'); /* found '\n' before the value */
    if( !ptr_start )
    {
        return -3;
    }
    ptr_start++;   /* skip '\n'  */

    /*+-----------------------+
     *|  Find end poistion  |
     *+-----------------------+
     */
    ptr_end=strstr(buf, "\r\n\r\nOK\r\n");
    if( !ptr_end )
    {
        return -3;
    }

    /*+-----------------------+
     *|    copy reply out     |
     *+-----------------------+
     */
    memset(reply, 0, size);
    while(ptr_start<ptr_end && i<size)
    {
        reply[i++] = *ptr_start;
        ptr_start ++;
    }

    return 0;
}


/*
 *  Description: Send AT command which will reply by the value  with a prefix "+CMD: " line, such as AT+CSQ:
 *                  Reply:   \r\n+CSQ: 26,99\r\nOK\r\n
 *
 * Return Value: 0: OK     -X: Failure
 */
int send_atcmd_check_request(char *atcmd, unsigned int timeout, char *reply, int size)
{
    int                     i = 0;
    int                     rv;
    char                    buf[ATCMD_REPLY_LEN];
    char                   *ptr_start, *ptr_end;

    if( !atcmd || !reply || size<=0 )
    {
    	atcmd_dbg("Invalid input arguments\r\n");
        return -1;
    }

    /*+-----------------------+
     *|    send AT command    |
     *+-----------------------+
     */
    rv = send_atcmd(atcmd, EXPECT_OK, timeout, buf, sizeof(buf));
    if( rv < 0 )
    {
    	atcmd_dbg("send_atcmd failure, rv=%d\r\n", rv);
        return -2;
    }


    /*+-----------------------+
     *|  Find start poistion  |
     *+-----------------------+
     */
    ptr_start=strchr(buf, '+');  /* found '+' before the value */
    if( !ptr_start )
    {
        return -3;
    }
    ptr_start++;   /* skip '+'  */

    ptr_start=strchr(buf, ':');  /* found '+' before the value */
    if( !ptr_start )
    {
        return -4;
    }
    ptr_start++;   /* skip ':'  */



    /*+-----------------------+
     *|  Find end poistion  |
     *+-----------------------+
     */
    ptr_end=strstr(buf, "\r\nOK\r\n");
    if( !ptr_end )
    {
        return -5;
    }

    if( *(ptr_end-1) == '\n' )
    {
    	ptr_end -= 2; /* remove previous \r\n */
    }

    /*+-----------------------+
     *|    copy reply out     |
     *+-----------------------+
     */
    memset(reply, 0, size);
    while(ptr_start<ptr_end && i<size)
    {
        reply[i++] = *ptr_start;
        ptr_start ++;
    }

    return 0;
}


int send_atcmd_check(int times)
{
	int        i;

	for(i=0; i<times; i++)
	{
		if( !send_atcmd_check_ok("AT\r", 500) )
		{
			atcmd_dbg("INFO: Send 'AT' to BC28 NB-IoT module and got reply ok\r\n");
			return 0;
		}

		HAL_Delay(100);
	}

	return -1;
}

int send_atcmd_echo(int status)
{
	int        mode = status ? 1 : 0;
	char       atcmd[64];

	snprintf(atcmd, sizeof(atcmd), "ATE%d\r", mode);

	if( !send_atcmd_check_ok(atcmd, 500) )
	{
		atcmd_dbg("INFO: Send 'ATE%d' to BC28 NB-IoT module and got reply ok\r\n", mode);
		return 0;
	}

	return -1;
}

