/**********************************************************************
*   Copyright: (C)2021 LingYun IoT System Studio <www.weike-iot.com>
*      Author: GuoWenxue<guowenxue@gmail.com> QQ: 281143292
* Description: BC28 NB-IoT module AT command source code
*
*
*   ChangeLog:
*        Version    Date       Author            Description
*        V1.0.0  2022.05.1  GuoWenxue      Release initial version
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bc28.h"

//#define CONFIG_BC28_DEBUG  /* AT命令相关函数调试开关 */

#ifdef CONFIG_BC28_DEBUG
#define bc28_dbg(format,args...) printf(format, ##args)
#else
#define bc28_dbg(format,args...) do{} while(0)
#endif


/* Check NB-IoT module name */
int atcmd_check_module_name(char *modname, int size)
{
	char        buf[256];
	int         bytes;


	if( 0 != send_atcmd_check_value("AT+CGMM\r", 500, buf, sizeof(buf)) )
	{
		bc28_dbg("ERROR: Send 'AT+CGMM' to query module name failure.\r\n");
		return -2;
	}

	bc28_dbg("Found NB-IoT module name '%s'\r\n", buf);

	if( modname && size>0 )
	{
		bytes = strlen(buf)>size ? size : strlen(buf);
		strncpy(modname, buf, bytes);
	}

	return 0;
}

/* Check NB-IoT module software version */
int atcmd_check_module_version(void)
{
	char        buf[256];


	if( 0 != send_atcmd_check_value("AT+CGMR\r", 500, buf, sizeof(buf)) )
	{
		bc28_dbg("ERROR: Send 'AT+CGMR' to query module software version failure.\r\n");
		return -2;
	}

	printf("Check NB-IoT module software version: \r\n'%s'\r\n", buf);

	return 0;
}


/* Check module IMEI(International Mobile station Equipment Identity) */
int atcmd_check_module_imei(char *imei, int size)
{
	char        buf[256];
	int         rv, bytes;


	rv=send_atcmd_check_request("AT+CGSN=1\r", 1000, buf, sizeof(buf));
	if( 0 != rv )
	{
		bc28_dbg("ERROR: Send 'AT+CGSN=1' to query module IMEI failure, rv=%d.\r\n", rv);
		return -2;
	}

	bc28_dbg("Check NB-IoT module IMEI '%s'\r\n", buf);

	if( imei && size>0 )
	{
		memset(imei, 0, size);
		bytes = strlen(buf) > size ? size : strlen(buf);
		strncpy(imei, buf, bytes);
	}

	return 0;
}


/* Check SIM card ready or not by check IMSI(International Mobile Subscriber Identity) */
int atcmd_check_sim_ready(void)
{
	char        buf[256] = {0x0};

	if( 0 != send_atcmd_check_value("AT+CIMI\r", 500, buf, sizeof(buf)) )
	{
		bc28_dbg("ERROR: Send 'AT+CIMI' to query module IMSI failure, SIM card not ready.\r\n");
		return -2;
	}

	printf("INFO: Check NB-IoT SIM Card IMSI   [%s]\r\n", buf);

	return 0;
}


/* Send AT command to reset NB-IoT module */
int atcmd_reset_module(void)
{
	if( !send_atcmd_check_ok("AT+NRB\r", 3000) )
	{
		bc28_dbg("INFO: Send 'AT+NRB' and got reply ok\r\n");
	}

	return 0;
}


/* Send AT command to set the UE Functionality. */
int atcmd_set_cfun(int fun)
{
	int        val = fun ? 1 : 0;
	char       atcmd[64] = {'\0'};

	snprintf(atcmd, sizeof(atcmd), "AT+CFUN=%d\r", val);

	if( !send_atcmd_check_ok(atcmd, 500) )
	{
		bc28_dbg("INFO: Send 'AT+CFUN=%d' and got reply ok\r\n", val);
		return 0;
	}

	return -1;
}

/* Send AT command to get the UE Functionality. */
int atcmd_get_cfun(void)
{
	char        buf[256];
	int         rv;

	rv=send_atcmd_check_request("AT+CFUN?\r", 1000, buf, sizeof(buf));
	if( 0 != rv )
	{
		bc28_dbg("ERROR: Send 'AT+CFUN?' to query module CFUN failure, rv=%d.\r\n", rv);
		return -2;
	}

	return atoi(buf);
}


/* Send AT command to set supported bands. */
int atcmd_set_bands(char *bands)
{
	char       atcmd[64] = {'\0'};

    if( !bands )
    {
    	bc28_dbg("Invalid input arguments\r\n");
        return -1;
    }

	snprintf(atcmd, sizeof(atcmd), "AT+NBAND=%s\r", bands);
	if( !send_atcmd_check_ok(atcmd, 500) )
	{
		bc28_dbg("INFO: Send 'AT+NBAND=%s' and got reply ok\r\n", bands);
		return 0;
	}

	bc28_dbg("INFO: Send 'AT+NBAND=%s' failed\r\n", bands);
	return -1;
}


/* Send AT command to get supported bands. */
int atcmd_get_bands(void)
{
	char        buf[256];
	int         rv;

	rv=send_atcmd_check_request("AT+NBAND?\r", 1000, buf, sizeof(buf));
	if( 0 != rv )
	{
		bc28_dbg("ERROR: Send 'AT+NBAND?' to query bands failure, rv=%d.\r\n", rv);
		return -2;
	}

	bc28_dbg("Check NB-IoT module band '%s'\r\n", buf);
	return 0;
}


/* Send AT command to set auto connect or not. */
int atcmd_set_autoconnect(int enable)
{
	char       atcmd[64] = {'\0'};


	snprintf(atcmd, sizeof(atcmd), "AT+NCONFIG=AUTOCONNECT,%s\r", enable?"TRUE":"FALSE");

	if( !send_atcmd_check_ok(atcmd, 500) )
	{
		bc28_dbg("INFO: Send 'AT+NCONFIG=AUTOCONNECT,%s' and got reply ok\r\n", enable?"TRUE":"FALSE");
		return 0;
	}

	bc28_dbg("ERROR: Send 'AT+NCONFIG=AUTOCONNECT,%s' failed\r\n", enable?"TRUE":"FALSE");
	return -1;
}

/* Send AT command to set NB-IoT network attach. */
int atcmd_set_attach(int attach)
{
	int        val = attach ? 1 : 0;
	char       atcmd[64] = {'\0'};

	snprintf(atcmd, sizeof(atcmd), "AT+CGATT=%d\r", val);

	if( !send_atcmd_check_ok(atcmd, 500) )
	{
		bc28_dbg("INFO: Send 'AT+CGATT=%d' and got reply ok\r\n", val);
		return 0;
	}

	bc28_dbg("ERROR: Send 'AT+CGATT=%d' failed.\r\n", val);
	return -1;
}

/* Send AT command to get NB-IoT network attach status. */
int atcmd_get_attach(void)
{
	char        buf[256];
	int         rv;

	rv=send_atcmd_check_request("AT+CGATT?\r", 1000, buf, sizeof(buf));
	if( 0 != rv )
	{
		bc28_dbg("ERROR: Send 'AT+CGATT?' to check NB-IoT attach failure, rv=%d.\r\n", rv);
		return -2;
	}

	rv = atoi(buf);
	bc28_dbg("INFO: Send 'AT+CGATT?' return [%d].\r\n", rv);

	return rv;
}


/* Send AT command to check NB-IoT network signal. */
int atcmd_check_sginal(void)
{
	char        buf[256];
	int         rv;

	rv=send_atcmd_check_request("AT+CSQ\r", 1000, buf, sizeof(buf));
	if( 0 != rv )
	{
		bc28_dbg("ERROR: Send 'AT+CSQ?' to check NB-IoT signal failure, rv=%d.\r\n", rv);
		return -2;
	}

	rv = atoi(buf);
	bc28_dbg("INFO: Send 'AT+CSQ?' return [%d].\r\n", rv);

	return rv;
}


/* Send AT command to check NB-IoT register status. */
int atcmd_check_register(void)
{
	char        buf[256];
	int         rv;
	char       *ptr;

	rv=send_atcmd_check_request("AT+CEREG?\r", 1000, buf, sizeof(buf));
	if( 0 != rv )
	{
		bc28_dbg("ERROR: Send 'AT+CEREG?' to check NB-IoT register failure, rv=%d.\r\n", rv);
		return -2;
	}

	ptr = strchr(buf, ',');
	if( !ptr )
	{
		return -3;
	}

	ptr++;

	rv = atoi(ptr);
	bc28_dbg("INFO: Send 'AT+CEREG?' return [%d]\r\n", rv);

	return rv;
}


/* Send AT command to check NB-IoT network IP address. */
int atcmd_check_ipaddr(char *ipaddr, int size)
{
	char        buf[256];
	int         rv;
	char       *ptr;

	rv=send_atcmd_check_request("AT+CGPADDR\r", 1000, buf, sizeof(buf));
	if( 0 != rv )
	{
		bc28_dbg("ERROR: Send 'AT+CGPADDR' to check NB-IoT IP address failure, rv=%d.\r\n", rv);
		return -2;
	}

	ptr = strchr(buf, ',');
	if( !ptr )
	{
		return -3;
	}

	ptr++;

	memset(ipaddr, 0, size);
	strncpy(ipaddr, ptr, size);

	bc28_dbg("INFO: Send 'AT+CGPADDR' got IP address [%s]\r\n", ipaddr);
	return 0;
}

/* Send AT command to check NB-IoT operator MCC number. */
int atcmd_check_operator(char *mcc, int size)
{
	char        buf[256];
	int         rv;
	char       *ptr;

	rv=send_atcmd_check_request("AT+COPS?\r", 1000, buf, sizeof(buf));
	if( 0 != rv )
	{
		bc28_dbg("ERROR: Send 'AT+COPS?' to check NB-IoT operator failure, rv=%d.\r\n", rv);
		return -2;
	}

	ptr = strrchr(buf, ',');
	if( !ptr )
	{
		return -3;
	}

	ptr++;

	memset(mcc, 0, size);
	strncpy(mcc, ptr, size);

	bc28_dbg("INFO: Send 'AT+COPS?' got operator MCC [%s].\r\n", mcc);
	return 0;
}


/*+--------------------------------------------------------+
 *|   NB-IoT module register to IoT platform functions     |
 *+--------------------------------------------------------+
 */

/* Send AT command to set register IoT platform server. */
int atcmd_set_iot_platform(char *server)
{
	char       atcmd[64] = {'\0'};

    if( !server )
    {
    	bc28_dbg("Invalid input arguments\r\n");
        return -1;
    }

	snprintf(atcmd, sizeof(atcmd), "AT+NCDP=%s\r", server);
	if( !send_atcmd_check_ok(atcmd, 500) )
	{
		bc28_dbg("INFO: Send 'AT+NCDP=%s' and got reply ok\r\n", server);
		return 0;
	}

	bc28_dbg("INFO: Send 'AT+NCDP=%s' failed\r\n", server);
	return -2;
}


/* Send AT command to check register IoT platform server. */
int atcmd_check_iot_platform(char *server)
{
	char        buf[256];
	int         rv;

    if( !server )
    {
    	bc28_dbg("Invalid input arguments\r\n");
        return -1;
    }

    rv=send_atcmd_check_request("AT+NCDP?\r", 1000, buf, sizeof(buf));
	if( rv != 0 )
	{
		bc28_dbg("INFO: Send 'AT+NCDP?' failed\r\n");
		return -2;
	}
	bc28_dbg("INFO: Send 'AT+NCDP?' and got reply ok\r\n");

	if( strstr(buf, server))
	{
		return 0;
	}
	else
	{
		printf("Not matched.\r\n");
	}

	return -3;
}


/* Send AT command to set data encrypto mode */
int atcmd_set_iot_encrypt_mode(int mode, int nat)
{
	char       atcmd[64] = {'\0'};

	if( MODE_ENCRYPT_DTLS_STD == mode )
		snprintf(atcmd, sizeof(atcmd), "AT+QSECSWT=%d,%d\r", mode, nat);
	else
		snprintf(atcmd, sizeof(atcmd), "AT+QSECSWT=%d\r", mode);

	if( !send_atcmd_check_ok(atcmd, 500) )
	{
		bc28_dbg("INFO: Send 'AT+QSECSWT=%d' and got reply ok\r\n", mode);
		return 0;
	}

	bc28_dbg("INFO: Send 'AT+QSECSWT=%d' failed\r\n", mode);
	return -1;
}


/* Send AT command to set IoT platform register mode */
int atcmd_set_iot_register_mode(int mode)
{
	char       atcmd[64] = {'\0'};

	snprintf(atcmd, sizeof(atcmd), "AT+QREGSWT=%d\r", mode);

	if( !send_atcmd_check_ok(atcmd, 500) )
	{
		bc28_dbg("INFO: Send 'AT+QREGSWT=%d' and got reply ok\r\n", mode);
		return 0;
	}

	bc28_dbg("INFO: Send 'AT+QREGSWT=%d' failed\r\n", mode);
	return -1;
}

/* Send AT command to set IoT platform register mode */
int atcmd_trigger_iot_register(int mode)
{
	char       atcmd[64] = {'\0'};

	snprintf(atcmd, sizeof(atcmd), "AT+QLWSREGIND=%d\r", mode);

	if( !send_atcmd_check_ok(atcmd, 500) )
	{
		bc28_dbg("INFO: Send 'AT+QLWSREGIND=%d' and got reply ok\r\n", mode);
		return 0;
	}

	bc28_dbg("INFO: Send 'AT+QLWSREGIND=%d' failed\r\n", mode);
	return -1;
}

/* Send AT command to query IoT platform register status */
int atcmd_get_iot_register_status(void)
{
	char        buf[256];
	int         rv = -3;

	rv=send_atcmd_check_request("AT+NMSTATUS?\r", 1000, buf, sizeof(buf));
	if( 0 != rv )
	{
		bc28_dbg("ERROR: Send 'AT+NMSTATUS?' to check IoT platform register failure, rv=%d.\r\n", rv);
		return -2;
	}

	if( strstr(buf, "MO_DATA_ENABLED"))
	{
		return 0;
	}

	return rv;
}


/* Send AT command to send data to IoT platform */
int atcmd_iot_send_data(char *data)
{
	char       atcmd[64] = {'\0'};
	int        len;

	if( !data || strlen(data)%2 )
	{
		bc28_dbg("Invalid input arguments\r\n");
		return -1;
	}

	len = strlen(data)/2;

	snprintf(atcmd, sizeof(atcmd), "AT+QLWULDATAEX=%d,%s,0X0100\r", len, data);

	if( !send_atcmd_check_ok(atcmd, 500) )
	{
		bc28_dbg("INFO: Send 'AT+QLWULDATAEX=%d,%s,0X0100' and got reply ok\r\n", len, data);
		return 0;
	}

	bc28_dbg("INFO: Send 'AT+QLWULDATAEX=%d,%s,0X0100' failed\r\n", len, data);
	return -2;
}
