/**********************************************************************
*   Copyright: (C)2021 LingYun IoT System Studio <www.weike-iot.com>
*      Author: GuoWenxue<guowenxue@gmail.com> QQ: 281143292
* Description: NB-IoT module source code
*
*
*   ChangeLog:
*        Version    Date       Author            Description
*        V1.0.0  2022.05.1  GuoWenxue      Release initial version
***********************************************************************/

#include <stdlib.h>
#include "nbiot.h"

#define CONFIG_NBIOT_DEBUG

#ifdef CONFIG_NBIOT_DEBUG
#define nbiot_dbg(format,args...) printf(format, ##args)
#else
#define nbiot_dbg(format,args...) do{} while(0)
#endif

int g_nbiot_status = NBIOT_STATUS_NOTREADY;
int g_nbiot_signal = 0;

int nbiot_module_init(void)
{
	int          rv;

	printf("Start to initial NB-IoT module...\r\n");

	rv = nbiot_set_manual_connect();
	if( 0 != rv )
	{
		printf("ERROR: Set NB-IoT manual connect   [Failed, rv=%d]\r\n", rv);
		return -2;
	}

	rv = nbiot_set_auto_connect();
	if( 0 != rv )
	{
		printf("ERROR: Set NB-IoT auto connect     [Failed, rv=%d]\r\n", rv);
		return -3;
	}

	rv = init_iot_platform();
	if( 0 != rv )
	{
		printf("ERROR: Set NB-IoT IoT platform     [Failed, rv=%d]\r\n", rv);
		return -4;
	}

	return 0;
}

int nbiot_check_module(void)
{
	int          rv;
	char         string[64];

	rv = send_atcmd_check(3);
	if( 0 != rv )
	{
			printf("ERROR: NB-IoT module AT command    [Not Ready]\r\n");
			rv = nbiot_module_reset();
			if( rv != 0 )
				return -2;
	}

	/* Set AT command echo off */
	send_atcmd_echo(ECHO_OFF);

	/* Check NB-IoT module name */
	rv = atcmd_check_module_name(string, sizeof(string));
	if( 0 != rv )
	{
		printf("ERROR: Check NB-IoT module name    [Failed, rv=%d]\r\n", rv);
		return -3;
	}
	printf("INFO: Found NB-IoT module name     [%s]\r\n", string);

	/* Check NB-IoT module IMEI */
	rv = atcmd_check_module_imei(string, sizeof(string));
	if( 0 != rv )
	{
		printf("ERROR: Check NB-IoT module IMEI    [Failed, rv=%d]\r\n", rv);
		return -4;
	}
	printf("INFO: Check NB-IoT module IMEI     [%s]\r\n", string);

	return 0;
}

int nbiot_module_reset(void)
{
	int          rv;

	nbiot_dbg("Start to reset NB-IoT module...\r\n");

	atcmd_reset_module();

	/* Check NB-IoT module ready or not */
	rv = send_atcmd_check(10);
	if( 0 != rv )
	{
		printf("ERROR: Reboot NB-IoT module        [Failed, rv=%d]\r\n", rv);
		return -1;
	}
	printf("INFO: Reboot NB-IoT module         [Ready]\r\n");
	return 0;
}


int nbiot_set_manual_connect(void)
{
	int          i, rv;

	rv = atcmd_set_autoconnect(0);
	if( rv )
	{
		printf("ERROR: Set NB-IoT manual connect   [Failed, rv=%d]\r\n", rv);
		return rv;
	}

	rv = atcmd_set_cfun(0);
	if( rv )
	{
		printf("ERROR: Disable NB-IoT RF           [Failed, rv=%d]\r\n", rv);

		return rv;
	}

	/* set china telecom */
	rv = atcmd_set_bands("1,3,5,8,20,28");
	if( rv )
	{
		printf("ERROR: Set NB-IoT bands            [Failed, rv=%d]\r\n", rv);
		return rv;
	}

	rv = nbiot_module_reset();
	if( rv )
	{
		printf("ERROR: Reboot NB-IoT module        [Failed, rv=%d]\r\n", rv);
		return rv;
	}

	for(i=0; i<10; i++)
	{
		rv = atcmd_set_cfun(1);
		if( 0 == rv )
		{
			break;
		}
		HAL_Delay(500);
	}

	if( 10 == i )
	{
		printf("ERROR: Enable NB-IoT RF            [Failed, rv=%d]\r\n", rv);
		return rv;
	}

	rv = atcmd_set_attach(1);
	if( rv )
	{
		printf("ERROR: Set NB-IoT attach           [Failed, rv=%d]\r\n", rv);
		return rv;
	}

	printf("INFO: Set NB-IoT manual connect    [Ready]\r\n");

	return 0;
}


int nbiot_set_auto_connect(void)
{
	int          rv;

	rv = atcmd_set_autoconnect(1);
	if( rv )
	{
		printf("ERROR: Set NB-IoT auto connect     [Failed, rv=%d]\r\n", rv);
		return rv;
	}

	rv = nbiot_module_reset();
	if( rv )
	{
		printf("ERROR: Reboot NB-IoT module        [Failed, rv=%d]\r\n", rv);
		return rv;
	}
	printf("INFO: Set NB-IoT auto connect      [Ready]\r\n");

	return 0;
}

int nbiot_check_status(void)
{
	int          rv;
	char         mcc[32];
	char         ipaddr[16];

	g_nbiot_status = NBIOT_STATUS_NOTREADY;
	g_nbiot_signal = 0;

	rv = atcmd_get_cfun();
	if( rv <= 0 )
	{
		printf("ERROR: Check NB-IoT CFUN           [Failed, rv=%d]\r\n", rv);
		return g_nbiot_status;
	}
	g_nbiot_status = NBIOT_STATUS_CFUN_OK;

	rv = atcmd_check_sim_ready();
	if( rv )
	{
		printf("ERROR: Check NB-IoT SIM Card       [Failed, rv=%d]\r\n", rv);
		return g_nbiot_status;
	}
	g_nbiot_status = NBIOT_STATUS_SIM_OK;

	rv = atcmd_check_sginal();
	if( rv<=0 || 99==rv )
	{
		printf("ERROR: Check NB-IoT signal         [Failed, rv=%d]\r\n", rv);
		return g_nbiot_status;
	}
	g_nbiot_signal = rv;
	printf("INFO: Check NB-IoT signal          [%d]\r\n", rv);


	rv = atcmd_check_register();
	if( 1==rv || 5==rv)
	{
		printf("INFO: Check NB-IoT register        [%d]\r\n", rv);
	}
	else
	{
		printf("ERROR: Check NB-IoT register       [Failed, rv=%d]\r\n", rv);
		return g_nbiot_status;
	}
	g_nbiot_status = NBIOT_STATUS_REG_OK;

	rv = atcmd_check_operator(mcc, sizeof(mcc));
	if( rv<0 )
	{
		printf("ERROR: Check NB-IoT operator       [Failed, rv=%d]\r\n", rv);
		return g_nbiot_status;
	}
	printf("INFO: Check NB-IoT operator MCC    [%s]\r\n", mcc);

	rv = atcmd_check_ipaddr(ipaddr, sizeof(ipaddr));
	if( rv<0 )
	{
		printf("ERROR: Check NB-IoT IP address     [Failed, rv=%d]\r\n", rv);
		return g_nbiot_status;
	}
	printf("INFO: Check NB-IoT IP address      [%s]\r\n", ipaddr);

	g_nbiot_status = NBIOT_STATUS_NET_OK;

	rv = atcmd_get_iot_register_status();
	if( rv != 0 )
	{
		printf("ERROR: Check IoT register status   [Failed, rv=%d]\r\n", rv);
		return g_nbiot_status;
	}
	printf("INFO: Check IoT register status    [Okay]\r\n");

	rv = atcmd_check_iot_platform(IOT_PLATFORM_SERVER);
	if( rv < 0 )
	{
		printf("ERROR: Check IoT platform server   [Failed, rv=%d]\r\n", rv);
		return g_nbiot_status;
	}
	printf("INFO: Check IoT platform server    [Okay]\r\n");

	g_nbiot_status = NBIOT_STATUS_IOT_OK;

	return g_nbiot_status;
}

int nbiot_update_status(void)
{
	int          rv;

	g_nbiot_signal = 0;
	rv = atcmd_check_sginal();
	if( rv<0 )
	{
		printf("ERROR: Check NB-IoT signal         [Failed, rv=%d]\r\n", rv);
		return rv;
	}
	g_nbiot_signal = rv;
	printf("INFO: Check NB-IoT signal:         [%d]\r\n", rv);

	rv = atcmd_check_register();
	if( 1!=rv && 5!=rv)
	{
		printf("ERROR: Check NB-IoT register       [Failed, rv=%d]\r\n", rv);
		g_nbiot_status = NBIOT_STATUS_NOTREADY;

		nbiot_module_init();
	}

	return 0;
}

int init_iot_platform(void)
{
	int          rv;

	rv = atcmd_set_iot_register_mode(MODE_REG_MANUAL);
	if( rv<0 )
	{
		printf("ERROR: Set IoT manual register     [Failed, rv=%d]\r\n", rv);
		return rv;
	}
	printf("INFO: Set IoT manual register      [Okay]\r\n");

	rv = nbiot_module_reset();
	if( rv )
	{
		printf("ERROR: Reboot NB-IoT module        [Failed, rv=%d]\r\n", rv);
		return rv;
	}

	rv = atcmd_set_iot_platform(IOT_PLATFORM_SERVER);
	if( rv<0 )
	{
		printf("ERROR: Set IoT platform server     [Failed, rv=%d]\r\n", rv);
		return rv;
	}
	printf("INFO: Set IoT platform server      [Okay]\r\n");

	rv = atcmd_set_iot_encrypt_mode(MODE_ENCRYPT_NONE, NAT_ENABLE);
	if( rv<0 )
	{
		printf("ERROR: Set IoT encrypt mode        [Failed, rv=%d]\r\n", rv);
		return rv;
	}
	printf("INFO: Set IoT encrypt mode         [Okay]\r\n");

	rv = atcmd_set_iot_register_mode(MODE_REG_AUTO);
	if( rv<0 )
	{
		printf("ERROR: Set IoT auto register       [Failed, rv=%d]\r\n", rv);
		return rv;
	}
	printf("INFO: Set IoT auto register        [Okay]\r\n");

	rv = nbiot_module_reset();
	if( rv )
	{
		printf("ERROR: Reboot NB-IoT module        [Failed, rv=%d]\r\n", rv);
		return rv;
	}

	return 0;
}
